package com.framsticks.leftovers.f0;

import java.util.Arrays;

import com.framsticks.params.FramsClass;
import com.framsticks.params.Param;

/**
 * The Class NeuroClass.
 */
public class NeuroClass {
	/*
	public String name, longname, description, code, icon;
	public Integer prefinputs, prefoutput, preflocation, vhints;
	*/

	/**
	 * The symbol glyph.
	 */
	private int[] symbolGlyph;

	/**
	 * The pref inputs.
	 */
	private int prefInputs;

	/**
	 * The pref output.
	 */
	private int prefOutput;

	/**
	 * The pref location.
	 */
	private int prefLocation;

	/**
	 * The param entry list.
	 */
	private FramsClass framsClass;

	/**
	 * The visual hints.
	 */
	private int visualHints;

	/**
	 * long description.
	 *
	 * @return the description
	 */
	String getDescription() {
		return framsClass.getDescription();
	}

	/**
	 * class getId (short getName).
	 *
	 * @return the getId
	 */
	String getId() {
		return framsClass.getId();
	}

	/**
	 * Instantiates a new neuro class.
	 *
	 * @param framsClass   the param entry list
	 * @param prefInputs   the pref inputs
	 * @param prefOutput   the pref output
	 * @param prefLocation the pref location
	 * @param visualHints  the visual hints
	 * @param symbolGlyph the symbol glymph
	 */
	public NeuroClass(FramsClass framsClass, int prefInputs,
					  int prefOutput, int prefLocation, int visualHints,
					  int[] symbolGlyph) {
		this.framsClass = framsClass;
		this.prefInputs = prefInputs;
		this.prefOutput = prefOutput;
		this.prefLocation = prefLocation;
		this.visualHints = visualHints;
		this.symbolGlyph = (symbolGlyph == null ? null : Arrays.copyOf(symbolGlyph, symbolGlyph.length));
	}

	/**
	 * human friendly getName, eg. "Neuron","Link","Gyroscope"
	 *
	 * @return the getName
	 */
	String getName() {
		return framsClass.getName();
	}

	/**
	 * NeuroClass specific properties, recognized by all neurons of this class.
	 *
	 * @return the param entry list
	 */
	FramsClass getFramsClass() {
		return framsClass;
	}

	/**
	 * Gets the param tab.
	 *
	 * @return the param tab
	 */
	FramsClass getParamTab() {
		return framsClass;
	}

	/**
	 * preferred number of inputs, -1 = no preference (any number will go).
	 * extra inputs may be ignored by the object (depends on the class).
	 *
	 * @return the preferred inputs
	 */
	int getPreferredInputs() {
		return prefInputs;
	}

	/**
	 * Gets the preferred location.
	 *
	 * @return 0 if the object doesn't need any assignment to the body element.
	 *         1 = it likes to be attached to the Part ( @see
	 *         Neuro::attachToPart() )
	 *         2 = the object prefers to have the Joint ( @see
	 *         Neuro::attachToJoint() )
	 */
	int getPreferredLocation() {
		return prefLocation;
	}

	/**
	 * Gets the preferred output.
	 *
	 * @return 0 if this object doesn't provide useful output signal.
	 */
	int getPreferredOutput() {
		return prefOutput;
	}

	/**
	 * vector drawing to be used in neuro net diagram. interpretation: { LEN =
	 * datalength (excluding this number) NL = number_of_lines line#1 -> NS =
	 * number_of_segments, x1,y1, x2,y2, ... xNS-1,yNS-1, ... line#NL -> NS =
	 * number_of_segments, x1,y1, x2,y2, ... xNS-1,yNS-1, }
	 *
	 * @return the symbol glyph
	 */
	int[] getSymbolGlyph() {
		return symbolGlyph;
	}

	/**
	 * vector drawing to be used in neuro net diagram. interpretation: { LEN =
	 * datalength (excluding this number) NL = number_of_lines line#1 -> NS =
	 * number_of_segments, x1,y1, x2,y2, ... xNS-1,yNS-1, ... line#NL -> NS =
	 * number_of_segments, x1,y1, x2,y2, ... xNS-1,yNS-1, }
	 *
	 * @param data the new symbol glyph
	 */
	void setSymbolGlyph(int[] data) {
		symbolGlyph = data;
	}

	/**
	 * additional information about how the neuron should be drawn used by
	 * structure view (and maybe some other components). return value is defined
	 * by the enum Hint
	 *
	 * @return the visual hints
	 */
	int getVisualHints() {
		return visualHints;
	}


	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("NeuroClass [getId=").append(framsClass.getId())
				.append(", name=").append(framsClass.getName())
				.append(", description=").append(framsClass.getDescription())
				.append(", groups#=").append(framsClass.getGroupCount())
				.append(", prefInputs=").append(prefInputs)
				.append(", prefOutput=").append(prefOutput)
				.append(", prefLocation=").append(prefLocation)
				.append(", visualHints=").append(visualHints)
				.append(", symbolGlymph=").append(Arrays.toString(symbolGlyph));

		sb.append(", paramList={");
		for (int i = 0; i < framsClass.getParamCount(); i++) {
			sb.append("\n");
			sb.append(framsClass.getParamEntry(i, Param.class).toString());
		}
		sb.append("\n}]");
		return sb.toString();
	}
}
